/*
 * Copyright (c) 2016 The CyanogenMod Project
 * Copyright (c) 2017-2022 The LineageOS Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.lineageos.settings.device.actions;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.telecom.TelecomManager;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;

import org.lineageos.settings.device.MotoActionsSettings;
import org.lineageos.settings.device.SensorHelper;

public class LiftToSilence extends PhoneStateListener implements SensorEventListener,
        UpdatedStateNotifier {
    private static final String TAG = "MotoActions-LiftToSilence";

    private final MotoActionsSettings mMotoActionsSettings;
    private final SensorHelper mSensorHelper;
    private final Sensor mFlatUpSensor;
    private final Sensor mStowSensor;

    private final TelecomManager mTelecomManager;
    private final TelephonyManager mTelephonyManager;

    private boolean mIsRinging;
    private boolean mIsStowed;
    private boolean mLastFlatUp;

    public LiftToSilence(MotoActionsSettings motoActionsSettings, Context context,
                         SensorHelper sensorHelper) {
        mMotoActionsSettings = motoActionsSettings;
        mSensorHelper = sensorHelper;
        mFlatUpSensor = sensorHelper.getFlatUpSensor();
        mStowSensor = sensorHelper.getStowSensor();
        mTelecomManager = (TelecomManager) context.getSystemService(Context.TELECOM_SERVICE);
        mTelephonyManager = context.getSystemService(TelephonyManager.class);
    }

    @Override
    public void updateState() {
        if (mMotoActionsSettings.isLiftToSilenceEnabled()) {
            mTelephonyManager.listen(this, LISTEN_CALL_STATE);
        } else {
            mTelephonyManager.listen(this, 0);
        }
    }

    @Override
    public synchronized void onCallStateChanged(int state, String incomingNumber) {
        if (state == TelephonyManager.CALL_STATE_RINGING && !mIsRinging) {
            Log.d(TAG, "Ringing started");
            mSensorHelper.registerListener(mFlatUpSensor, this);
            mSensorHelper.registerListener(mStowSensor, mStowListener);
            mIsRinging = true;
        } else if (state != TelephonyManager.CALL_STATE_RINGING && mIsRinging) {
            Log.d(TAG, "Ringing stopped");
            mSensorHelper.unregisterListener(this);
            mSensorHelper.unregisterListener(mStowListener);
            mIsRinging = false;
        }
    }


    @Override
    public synchronized void onSensorChanged(SensorEvent event) {
        boolean thisFlatUp = (event.values[0] != 0);

        Log.d(TAG, "event: " + thisFlatUp + " mLastFlatUp=" + mLastFlatUp + " mIsStowed=" +
                mIsStowed);

        if (mLastFlatUp && !thisFlatUp && !mIsStowed) {
            mTelecomManager.silenceRinger();
        }
        mLastFlatUp = thisFlatUp;
    }

    @Override
    public void onAccuracyChanged(Sensor mSensor, int accuracy) {
    }

    private final SensorEventListener mStowListener = new SensorEventListener() {
        @Override
        public synchronized void onSensorChanged(SensorEvent event) {
            mIsStowed = (event.values[0] != 0);
        }

        @Override
        public void onAccuracyChanged(Sensor mSensor, int accuracy) {
        }
    };
}
